package com.example.mynotepad.MenuFeatures.AllNotes;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.example.mynotepad.MenuFeatures.AllNotes.MyNoteRecyclerView.MyAllNotesAdaptor;
import com.example.mynotepad.MenuFeatures.AllNotes.MyNoteRecyclerView.MyNote;
import com.example.mynotepad.MenuFeatures.AllNotes.MyNoteRecyclerView.OnAllNotesItemClickListener;
import com.example.mynotepad.MenuFeatures.AllNotes.Note.NoteFragment;
import com.example.mynotepad.MenuFeatures.AllNotes.DataBase.Note;
import com.example.mynotepad.MenuFeatures.CustomToolbarOption;
import com.example.mynotepad.MenuFeatures.CustomToolbarOptionListener;
import com.example.mynotepad.R;
import com.google.android.material.floatingactionbutton.FloatingActionButton;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import static com.example.mynotepad.MainActivity.noteDataBase;

public class AllNotesActivity extends AppCompatActivity {

    private FrameLayout fragPlace;
    private LinearLayout allNoteListAndTitle;
    private FloatingActionButton addNoteButton;
    private FragmentTransaction fragmentTransaction;
    private CustomToolbarOption customToolbarOption;
    private FrameLayout allNoteToolBar;
    private RecyclerView recyclerView;
    private MyAllNotesAdaptor myAdaptor;
    private List<Note> entityNotes;
    private List<MyNote> myNotes;
    private static MyNote myChosenNote;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_all_notes);
        init();
    }

    private void init() {
        // get data from database
        setAllNoteListAndTitles();

        customToolbarOption = findViewById(R.id.customToolbarOption);
        customToolbarOption.setVisibility(View.GONE);
        allNoteToolBar = findViewById(R.id.allNotesToolbar);
        allNoteToolBar.setEnabled(false);
        addNoteButton = findViewById(R.id.addNoteButton);
        allNoteListAndTitle = findViewById(R.id.allNoteListAndTitle);
        fragPlace = findViewById(R.id.fragmentPlace);
        fragPlace.setVisibility(View.GONE);


        // show list with recycler view
        recyclerView = findViewById(R.id.recyclerView);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false);
        myAdaptor = new MyAllNotesAdaptor(myNotes);

        recyclerView.setLayoutManager(linearLayoutManager);
        recyclerView.setAdapter(myAdaptor);
        addListeners();

    }


    private void addListeners() {
        addNoteButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                showAddAlert();
            }
        });

        // add listener to recyclerView
        myAdaptor.setOnItemClickListener(new OnAllNotesItemClickListener() {
            @Override
            public void onItemClicked(MyNote myNote) {
                myChosenNote = myNote;
                //set noteFragment data
                final Bundle bundle = new Bundle();
                bundle.putString("title", myNote.getTitle());
                bundle.putString("bodyTxt", myNote.getDescription());
                final NoteFragment newNoteFrag = new NoteFragment(false);
                newNoteFrag.setArguments(bundle);

                //change the fragment
                fragPlace.setVisibility(View.VISIBLE);
                allNoteListAndTitle.setVisibility(View.GONE);
                addNoteButton.hide();
                replaceFragment(newNoteFrag);
            }

            @Override
            public void onItemLongClicked(MyNote myNote) {
                customToolbarOption.setVisibility(View.VISIBLE);
                allNoteToolBar.setVisibility(View.GONE);
                myChosenNote = myNote;
                if (myChosenNote.isarchived()){
                    customToolbarOption.setYellowStar();
                }
                else {
                    customToolbarOption.setBlackStar();
                }
            }
        });

        //add listener to custom toolbar option and set being archived or not to be saved in database
        customToolbarOption.setCustomToolbarOptionListener(new CustomToolbarOptionListener() {
            @Override
            public void onStarClicked() {
                starClicked();
            }

            @Override
            public void onDeleteClicked() {
                Note note = entityNotes.get(myNotes.indexOf(myChosenNote));
                deleteClicked(note);
            }

            @Override
            public void onCloseClicked() {
                closeClicked(customToolbarOption);
            }
        });
    }

    protected void closeClicked(CustomToolbarOption customToolbarOption) {
        customToolbarOption.setVisibility(View.GONE);
        allNoteToolBar.setVisibility(View.VISIBLE);
    }

    protected void deleteClicked(final Note note) {
        AlertDialog.Builder alert = new AlertDialog.Builder(AllNotesActivity.this);
        alert.setMessage("are you sure you wanna delete it ??? ");
        alert.setPositiveButton("yes", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                noteDataBase.noteDao().deleteNote(note);
                init();
            }
        });
        alert.setNegativeButton("no", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                dialogInterface.cancel();
            }
        });
        alert.show();
        allNoteToolBar.setVisibility(View.VISIBLE);
    }

    protected void starClicked() {
        if (myChosenNote.isarchived()) {
            customToolbarOption.setBlackStar();
            myChosenNote.setarchived(false);
            updatearchivedNotesInDataBase();
            Toast.makeText(this, "unarchived", Toast.LENGTH_SHORT).show();
        } else {
            customToolbarOption.setYellowStar();
            myChosenNote.setarchived(true);
            updatearchivedNotesInDataBase();
            Toast.makeText(this, "archived", Toast.LENGTH_SHORT).show();
        }
    }

    private void updatearchivedNotesInDataBase() {
        noteDataBase.noteDao().updateNote(myChosenNote.getTitle(), myChosenNote.getDescription()
                , myChosenNote.isarchived(), myChosenNote.getId());
    }


    private void showAddAlert() {
        final AlertDialog.Builder alert = new AlertDialog.Builder(this);
        alert.setTitle("new note :");
        alert.setMessage("Do you want to create a new note ?");
        alert.setCancelable(true);
        alert.setPositiveButton("yes", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                showABlankNote();
            }
        });
        alert.setNegativeButton("no", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                dialogInterface.cancel();
            }
        });
        alert.show();
    }

    private void showABlankNote() {
        fragPlace.setVisibility(View.VISIBLE);
        allNoteListAndTitle.setVisibility(View.GONE);
        addNoteButton.hide();

        NoteFragment newNoteFrag = new NoteFragment(true);
        replaceFragment(newNoteFrag);
    }

    public void replaceFragment(Fragment fragment) {
        fragmentTransaction = getSupportFragmentManager().beginTransaction();
        fragmentTransaction.replace(fragPlace.getId(), fragment);
        fragmentTransaction.commit();
    }


    private void setAllNoteListAndTitles() {
        entityNotes = noteDataBase.noteDao().getAll();
        myNotes = new ArrayList<>();
        for (Note note : entityNotes) {
            MyNote myNote = new MyNote(note.getTitle(), note.getDescription(), note.isarchived(), note.getID());
            myNotes.add(myNote);
        }
    }

}