package com.example.mynotepad.MenuFeatures.Calender;

import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.example.mynotepad.MenuFeatures.AllNotes.DataBase.DateEntity;
import com.example.mynotepad.MenuFeatures.AllNotes.DataBase.Note;
import com.example.mynotepad.MenuFeatures.AllNotes.Note.NoteFragment;
import com.example.mynotepad.MenuFeatures.Calender.MyCalenderRecyclerView.MyCalenderAdaptor;
import com.example.mynotepad.MenuFeatures.Calender.MyCalenderRecyclerView.OnCalenderItemClickListener;
import com.example.mynotepad.MenuFeatures.Calender.MyCalenderRecyclerView.TimePicker.DatePickerActivity;
import com.example.mynotepad.MenuFeatures.Calender.MyCalenderRecyclerView.TimePicker.MyDate;
import com.example.mynotepad.MenuFeatures.CustomToolbarOption;
import com.example.mynotepad.MenuFeatures.CustomToolbarOptionListener;
import com.example.mynotepad.R;
import com.google.android.material.floatingactionbutton.FloatingActionButton;

import java.util.ArrayList;
import java.util.List;

import static com.example.mynotepad.MainActivity.noteDataBase;


public class MyCalenderActivity extends AppCompatActivity {

    private FloatingActionButton addDateBtn;
    private CustomToolbarOption customToolbarOption;
    private FrameLayout calenderToolbar;
    private LinearLayout allDateListAndTitle;
    private FrameLayout fragPlace;
    private RecyclerView recyclerView;
    private MyCalenderAdaptor myAdaptor;
    private List<MyDate> myDates;
    private List<DateEntity> entityDates;
    private MyDate myChosenDateNote;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_my_calender);
        init();
    }

    @Override
    protected void onResume() {
        super.onResume();
        init();
    }

    private void init() {

        setAllDateNoteListAndTitles();
        addDateBtn = findViewById(R.id.addDateButton);

        customToolbarOption = findViewById(R.id.calenderCustomToolbarOption);
        customToolbarOption.setVisibility(View.GONE);
        calenderToolbar = findViewById(R.id.calenderToolbar);
        calenderToolbar.setEnabled(false);
        addDateBtn = findViewById(R.id.addDateButton);
        allDateListAndTitle = findViewById(R.id.allDateListAndTitle);
        fragPlace = findViewById(R.id.fragmentPlace_calender);
        fragPlace.setVisibility(View.GONE);


        // show list with recycler view
        recyclerView = findViewById(R.id.calenderRecyclerView);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false);
        myAdaptor = new MyCalenderAdaptor(myDates);
        recyclerView.setAdapter(myAdaptor);
        recyclerView.setLayoutManager(linearLayoutManager);

        addListeners();
    }

    private void addListeners() {
        addDateBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                addNewDateAndDescription();
            }
        });

        // add listener to recyclerView
        myAdaptor.setOnCalenderItemClickListener(new OnCalenderItemClickListener() {
            @Override
            public void onItemClicked(MyDate myDate) {
                myChosenDateNote = myDate;
                //set noteFragment data
                final Bundle bundle = new Bundle();
                bundle.putString("title", myChosenDateNote.getDate());
                bundle.putString("bodyTxt", myChosenDateNote.getDescription());
                final NoteFragment newNoteFrag = new NoteFragment(false);
                newNoteFrag.setArguments(bundle);

                //change the fragment
            /*    fragPlace.setVisibility(View.VISIBLE);
                allNoteListAndTitle.setVisibility(View.GONE);
                addNoteButton.hide();
                replaceFragment(newNoteFrag);*/
                //toDo go to activity/fragment
            }

            @Override
            public void onItemLongClicked(MyDate myDate) {
                customToolbarOption.setVisibility(View.VISIBLE);
                calenderToolbar.setVisibility(View.GONE);
                myChosenDateNote = myDate;
                if (myChosenDateNote.isarchived()){
                    customToolbarOption.setYellowStar();
                }
                else {
                    customToolbarOption.setBlackStar();
                }
            }
        });


        //add listener to custom toolbar option and set being archived or not to be saved in database
        customToolbarOption.setCustomToolbarOptionListener(new CustomToolbarOptionListener() {
            @Override
            public void onStarClicked() {
                starClicked();
            }

            @Override
            public void onDeleteClicked() {
                deleteClicked();

            }

            @Override
            public void onCloseClicked() {
                closeClicked();
            }
        });
    }


    private void closeClicked() {
        customToolbarOption.setVisibility(View.GONE);
        calenderToolbar.setVisibility(View.VISIBLE);
    }

    private void deleteClicked() {
        AlertDialog.Builder alert = new AlertDialog.Builder(MyCalenderActivity.this);
        alert.setMessage("are you sure you wanna delete it ??? ");
        alert.setPositiveButton("yes", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                noteDataBase.dateNoreDAO().deleteDateNote(entityDates.get(myDates.indexOf(myChosenDateNote)));
                init();
            }
        });
        alert.setNegativeButton("no", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                dialogInterface.cancel();
            }
        });
        alert.show();
        calenderToolbar.setVisibility(View.VISIBLE);
    }

    private void starClicked() {
        if (myChosenDateNote.isarchived()) {
            customToolbarOption.setBlackStar();
            myChosenDateNote.setarchived(false);
            updatearchivedDatesInDataBase();
            Toast.makeText(this, "unarchived", Toast.LENGTH_SHORT).show();
        } else {
            customToolbarOption.setYellowStar();
            myChosenDateNote.setarchived(true);
            updatearchivedDatesInDataBase();
            Toast.makeText(this, "archived", Toast.LENGTH_SHORT).show();
        }
    }

    private void updatearchivedDatesInDataBase() {
        noteDataBase.dateNoreDAO().updateDate(myChosenDateNote.getDate(), myChosenDateNote.getDescription(), myChosenDateNote.isarchived(), myChosenDateNote.getId());
    }


    private void addNewDateAndDescription() {
        Intent intent = new Intent(MyCalenderActivity.this, DatePickerActivity.class);
        startActivity(intent);
    }

    private void setAllDateNoteListAndTitles() {
        //get data from database to set in recyclerView
        entityDates = noteDataBase.dateNoreDAO().getAllDateNotes();
        myDates = new ArrayList<>();
        for (DateEntity dateEntity : entityDates) {
            MyDate myDate = new MyDate(dateEntity.getDate(), dateEntity.getDescription(), dateEntity.isarchived(), dateEntity.getID());
            myDates.add(myDate);
        }
    }

}