package com.example.mynotepad.MenuFeatures.VoiceMassages.VoiceMassagesRecyclerView;

import android.content.Context;
import android.media.MediaPlayer;
import android.os.Build;
import android.os.Handler;
import android.transition.TransitionManager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.recyclerview.widget.RecyclerView;

import com.example.mynotepad.R;

import java.io.IOException;
import java.util.ArrayList;

public class VoiceAdaptor extends RecyclerView.Adapter<VoiceAdaptor.VoiceViewHolder> {
    private Context context;
    private ArrayList<MyVoice> myVoiceArrayList;
    private MediaPlayer mPlayer;
    private boolean isPlaying = false;
    private int last_index = -1;
    private OnVoiceMassageClickListener onVoiceMassageClickListener;

    public VoiceAdaptor(Context context, ArrayList<MyVoice> myVoiceArrayList) {
        this.context = context;
        this.myVoiceArrayList = myVoiceArrayList;
    }


    @NonNull
    @Override
    public VoiceViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.custom_voice_item, parent, false);
        return new VoiceViewHolder(view);
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    @Override
    public void onBindViewHolder(@NonNull VoiceViewHolder holder, int position) {
        setUpData(holder, position);
    }


    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    private void setUpData(VoiceViewHolder holder, int position) {

        MyVoice myVoice = myVoiceArrayList.get(position);
        holder.textViewName.setText(myVoice.getFileName());

        if (myVoice.isPlaying()) {
            holder.imageViewStop.setVisibility(View.VISIBLE);
            holder.imageViewPlay.setVisibility(View.GONE);
            TransitionManager.beginDelayedTransition((ViewGroup) holder.itemView);
            holder.seekBar.setVisibility(View.VISIBLE);
            holder.seekUpdation(holder);
        } else {
            holder.imageViewStop.setVisibility(View.GONE);
            holder.imageViewPlay.setVisibility(View.VISIBLE);
            TransitionManager.beginDelayedTransition((ViewGroup) holder.itemView);
            holder.seekBar.setVisibility(View.GONE);
        }

        holder.manageSeekBar(holder);

    }

    @Override
    public int getItemCount() {
        return myVoiceArrayList.size();
    }

    public void setOnVoiceMassageClickListener(OnVoiceMassageClickListener onVoiceMassageClickListener) {
        this.onVoiceMassageClickListener = onVoiceMassageClickListener;
    }

    class VoiceViewHolder extends RecyclerView.ViewHolder implements View.OnLongClickListener {
        FrameLayout startStopBtn;
        ImageView imageViewPlay, imageViewStop;
        SeekBar seekBar;
        TextView textViewName;
        private String recordingUri;
        private Handler mHandler = new Handler();
        VoiceViewHolder holder;

        public VoiceViewHolder(View itemView) {
            super(itemView);

            imageViewPlay = itemView.findViewById(R.id.playImg);
            imageViewStop = itemView.findViewById(R.id.pauseImg);
            seekBar = itemView.findViewById(R.id.seekBar);
            textViewName = itemView.findViewById(R.id.voiceName);
            startStopBtn = itemView.findViewById(R.id.start_stopBtn);
            imageViewStop.setVisibility(View.GONE);
            itemView.setOnLongClickListener(this);

            /*
              add clickListener to stop or start playing
             */
            startStopBtn.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {

                    int position = getAdapterPosition();
                    MyVoice myVoice = myVoiceArrayList.get(position);
                    recordingUri = myVoice.getUri();

                    //if media player is playing
                    if (isPlaying) {
                        stopPlaying();
                        //if another clicked item (your last chosen item) is playing and you click on it again to stop it
                        if (position == last_index) {
                            myVoice.setPlaying(false);
                            stopPlaying();
                            notifyItemChanged(position);
                        }
                        // if you click on another item while your last one(your last chosen item) is playing
                        else {
                            markAllPaused();
                            myVoice.setPlaying(true);
                            notifyItemChanged(position);
                            startPlaying(myVoice, position);
                            last_index = position;// set the last playing item on the one which is playing
                        }
                    }
                    //if media player is not playing
                    else {
                        //if clicked item is playing it is wrong so it should be stopped
                        if (myVoice.isPlaying()) {
                            myVoice.setPlaying(false);
                            stopPlaying();
                            Log.d("isPlayin", "True");
                        }
                        //if nothing is playing and you want to play a new voice massage
                        else {
                            startPlaying(myVoice, position);
                            myVoice.setPlaying(true);
                            seekBar.setMax(mPlayer.getDuration());
                            Log.d("isPlayin", "False");
                        }
                        notifyItemChanged(position);
                        last_index = position;// set the last playing item on the one which is playing
                    }

                }

            });
        }

        public void manageSeekBar(VoiceViewHolder holder) {
            holder.seekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
                @Override
                public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                    if (mPlayer != null && fromUser) {
                        mPlayer.seekTo(progress);
                    }
                }

                @Override
                public void onStartTrackingTouch(SeekBar seekBar) {
                }

                @Override
                public void onStopTrackingTouch(SeekBar seekBar) {
                }
            });
        }



        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                seekUpdation(holder);
            }
        };

        private void seekUpdation(VoiceViewHolder holder) {
            this.holder = holder;
            if (mPlayer != null) {
                int mCurrentPosition = mPlayer.getCurrentPosition();
                holder.seekBar.setMax(mPlayer.getDuration());
                holder.seekBar.setProgress(mCurrentPosition);
            }
            mHandler.postDelayed(runnable, 100);
        }

        private void stopPlaying() {
            try {
                mPlayer.release();
            } catch (Exception e) {
                e.printStackTrace();
            }
            mPlayer = null;
            isPlaying = false;
        }

        private void startPlaying(final MyVoice audio, final int position) {
            mPlayer = new MediaPlayer();
            try {
                mPlayer.setDataSource(recordingUri);
                mPlayer.prepare();
                mPlayer.start();
            } catch (IOException e) {
                Log.e("LOG_TAG", "prepare() failed");
            }
            //showing the pause button
            seekBar.setMax(mPlayer.getDuration());
            isPlaying = true;

            mPlayer.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {
                @Override
                public void onCompletion(MediaPlayer mp) {
                    audio.setPlaying(false);
                    notifyItemChanged(position);
                }
            });


        }

        @Override
        public boolean onLongClick(View view) {
            if (onVoiceMassageClickListener != null)
                onVoiceMassageClickListener.onItemLongClicked(myVoiceArrayList.get(getAdapterPosition()));
            return true;
        }
    }
    /*
     * set all voice items on pause and stop all of them
     */
    private void markAllPaused() {
        for (int i = 0; i < myVoiceArrayList.size(); i++) {
            myVoiceArrayList.get(i).setPlaying(false);
            myVoiceArrayList.set(i, myVoiceArrayList.get(i));
        }
        notifyDataSetChanged();
    }
}
